/*
 * Aspera, Inc.
 *
 * C++ only.
 *
 * FaspStreamException class.
 */
#pragma once

#include <string>
#include <vector>
#include <Poco/String.h>

class DebugInfo
{
private:
    std::string _executable;
    std::string _arguments;

public:
    DebugInfo(const std::string executable,
              const std::vector<std::string> arguments) :
        _executable(executable) {

        _arguments = Poco::cat(std::string(" "), arguments.begin(),
            arguments.end());

    }

    DebugInfo(const DebugInfo &di) :
        _executable(di._executable),
        _arguments(di._arguments) { }

    DebugInfo() :
        _executable("faspstream"),
        _arguments("") {}

    std::string getCommonIssueMessage() const {
        std::string message("Please check to ensure that:\n");
        message.append("     You have a valid aspera.conf file\n");
        message.append("     You have a valid Aspera Client license\n");
        message.append("     You have installed ");
        message.append(_executable);
        message.append(" properly\n");
        message.append("     Your firewall is not blocking your TCP or UDP ports\n");
        message.append("     Command-line arguments: \n");
        message.append(_arguments);

        return message;
    }

};

/**
 * Thrown when various runtime exceptions and errors occur in the internal
 * code.
 */
class FaspStreamException : public std::exception {
public:
    FaspStreamException(const std::string& what, const DebugInfo &info) :
        _what(what), _details(info) {}
    FaspStreamException(const std::string& what) : _what(what) {}
    FaspStreamException(const FaspStreamException &ex) :
        _what(ex._what), _details(ex._details) {}
    virtual ~FaspStreamException() throw() {}
    /*
     * Info
     */
    virtual const char* what() const throw() { return _what.c_str(); }
    virtual std::string getDebugDetails() const {
        return _details.getCommonIssueMessage();
    }
private:
    std::string _what;
    DebugInfo _details;
};
